//-------------------------------------------------------------------------
// NEOFOX Information ID's (for use with DeviceGetParameter)
//-------------------------------------------------------------------------
#ifndef _NEOFOX_PROTOCOL_H
#define _NEOFOX_PROTOCOL_H

// ID Information
#define NEOFOX_ID                       0        // Target Serial Number
#define NEOFOX_NAME                     1        // Target Friendly Name
#define NEOFOX_FIRMWARE_VER             2        // Target ARM Main Processor Revision
#define NEOFOX_USB_DEVICE_NUM           3        // USB Device Number
#define NEOFOX_FLASH_WR_CNT             4        // Number of time flash memory has been written
#define NEOFOX_SERIAL_NUMBER            5        // DS2401 Hardware Serial Number - 8 bytes
#define NEOFOX_DISPLAY_STATE            6        // Current display state of NEOFOX

// Status Information
#define NEOFOX_TEMPERATURE             10        // Temperature sensor reading
#define NEOFOX_VREF_SENSOR             11        // Voltage for sensor reference
#define NEOFOX_TEMPERATURE_BRD1        12        // Board Temperature Readings
#define NEOFOX_TEMPERATURE_BRD2        13
#define NEOFOX_TEMPERATURE_BRD3        14
#define NEOFOX_ADC_PRESSURE            15        // Air pressure reading
#define NEOFOX_ADC_BOARD_VOLTAGE       16        // Measured 5.5V power supply level
#define NEOFOX_ADC_APD_VOLTAGE         17        // Measured power supply level of APD supply
#define NEOFOX_FPGA_STATUS             18        // Status register of FPGA
#define NEOFOX_TAU                     19        // Calculated TAU
#define NEOFOX_OXYGEN                  20        // Calculated O2 level
#define NEOFOX_DATA_STATUS             21        // Data integrity check
#define NEOFOX_ADC_BATTERY_VOLTAGE     22        // Measured battery power supply level
#define NEOFOX_OXYGEN_CONVERTED        23        // Calculated oxygen in alternate units

// Red LED
#define NEOFOX_RED_PHASE               30        // Phase calculation
#define NEOFOX_RED_PHASE_RAW           31        // Most recent phase calculation
#define NEOFOX_RED_DECAY_TIME          32        // Waveform decay time
#define NEOFOX_RED_SIGNAL_QUALITY      33        // Waveform signal quality
#define NEOFOX_RED_AMPLITUDE           34        // Waveform amplitude
#define NEOFOX_RED_AMPLITUDE_LO        35        // Waveform low point
#define NEOFOX_RED_AMPLITUDE_HI        36        // Waveform high point
#define NEOFOX_RED_PEAK                37        // Peak high point
#define NEOFOX_RED_PEAK_LO             38        // Peak amplitude
#define NEOFOX_RED_PEAK_HI             39        // Peak low point

// Blue LED
#define NEOFOX_BLUE_PHASE              50        // Phase calculation
#define NEOFOX_BLUE_PHASE_RAW          51        // Most recent phase calculation
#define NEOFOX_BLUE_DECAY_TIME         52        // Waveform decay time
#define NEOFOX_BLUE_SIGNAL_QUALITY     53        // Waveform signal quality
#define NEOFOX_BLUE_AMPLITUDE          54        // Waveform amplitude
#define NEOFOX_BLUE_AMPLITUDE_LO       55        // Waveform low point
#define NEOFOX_BLUE_AMPLITUDE_HI       56        // Waveform high point
#define NEOFOX_BLUE_PEAK               57        // Peak high point
#define NEOFOX_BLUE_PEAK_LO            58        // Peak amplitude
#define NEOFOX_BLUE_PEAK_HI            59        // Peak low point
                                       
#define NEOFOX_DUTY_CYCLE_SEC_CNT      65        // Incrementing second count for duty cycling
#define NEOFOX_DUTY_CYCLE_SEC_TICK     66        // Flag that we just hit a second boundary
#define NEOFOX_DUTY_CYCLE_LED_STATE    67        // On / Off state of duty cycle

#define NEOFOX_APD_AUTOSET_STATE       70        // Current APD auto gain status (Read Only)
#define NEOFOX_RUN_START_TIME          71        // Most recent run start time (Read Only)
#define NEOFOX_RUN_STOP_TIME           72        // Most recent run stop  time (Read Only)
#define NEOFOX_RUN_TIME                73        // Time device has been running (Read Only)
#define NEOFOX_UP_TIME                 74        // Number of milliseconds NEOFOX has been powered on (READ ONLY)
#define NEOFOX_HOST_DROP               75        // Number of host commands dropped due to overflow
#define NEOFOX_HOST_FREE               76        // Max allowed host command number

// Misc.
#define NEOFOX_ANALOG_OUT_4_20_MA      82        // Analog setting of 4 to 20 ma board output
#define NEOFOX_ANALOG_OUT_5V           83        // Analog setting of 0 to 5V board output

#define NEOFOX_POST_RESULTS            84        // Power on self test results

// Target Settable Parameters
#define NEOFOX_PING                    90        // Message to indicate active PC connection
#define NEOFOX_DIAGNOSTICS_LED         91        // Diagnostics LED
#define NEOFOX_FLASH_RESET             92        // Reset Flash Parameters To Defaults
#define NEOFOX_FLASH_WRITE             93        // Write all non-volatile parameters to flash
#define NEOFOX_WRITE_JAZ               94        // Write protocol to Jaz interface
#define NEOFOX_LCD_ENABLE              95        // Enable use of the LCD controller for display

// Sensor Control
#define NEOFOX_APD_AUTOSET_START      100        // Begin a APD voltage autogain sequence
#define NEOFOX_AUTOGAIN_ENABLE        101        // Enable autogain sequence
#define NEOFOX_BLUE_LED_PGA_GAIN      102        // Programmable gain amplifier - Blue LED
#define NEOFOX_BLUE_GAIN_1ST_STAGE    103        // Gain level to use for first stage - Blue LED
#define NEOFOX_FPGA_RESET             104        // Perform a reset sequence on the FPGA
#define NEOFOX_RED_LED_PGA_GAIN       105        // Programmable gain amplifier - Red LED
#define NEOFOX_AUTOGAIN_LEVEL_LO      106        // Lowest allowed level before autogain occurs
#define NEOFOX_AUTOGAIN_LEVEL_HI      107        // Highest allowed level before autogain occurs
#define NEOFOX_AUTOGAIN_AMPL_LO       108        // Lowest amplitude allowed before autogain occurs
#define NEOFOX_AUTOGAIN_AMPL_HI       109        // Highest amplitude allowed before autogain occurs
#define NEOFOX_SENSOR_AUTOSET         110        // Set default settings based on sensor type
#define NEOFOX_RED_GAIN_1ST_STAGE     111        // Gain level to use for first stage - Red LED

#define NEOFOX_DUTY_CYCLE_SEC_ON      118        // LED duty cycling, number of seconds to stay on
#define NEOFOX_DUTY_CYCLE_SEC_OFF     119        // LED duty cycling, number of seconds to remain off
#define NEOFOX_LED_SEQUENCE_ON        120        // Control LED sequencing on / off
#define NEOFOX_LED_CONTROL            121        // Misc. LED control 
#define NEOFOX_ADC_CONTROL            122        // Misc. ADC control = AdcOEn,  [Gain * 1.5],  Rand, Mode (0/1), Dither,  Shutdown
#define NEOFOX_ADC_CLK_DIVIDER        123        // Divisor for main clock to create system clock
#define NEOFOX_LED_ON_TIME            124        // # of clocks to hold LED high
#define NEOFOX_LED_CYCLE_TIME         125        // # of clocks for entire LED cycle 

#define NEOFOX_LED_REC_TIME           127        // # of ADC samples to record (power of 2 encoded)
#define NEOFOX_LED_AVG_CYCLES         128        // # LED cycles to average 
#define NEOFOX_PHASE_AVG_CNT_BLUE     129        // # phase results to average for blue LED
#define NEOFOX_SENSOR_DATA_TYPE       130        // Simulated waveforms
#define NEOFOX_BASELINE_PHASE         131        // Set baseline phase (subtracted from computed phase)
#define NEOFOX_FPGA_RESTART           132        // Reset FPGA and re-write registers
#define NEOFOX_LED_CLOCK_RATE         133        // Set clock rate, optimize settings
#define NEOFOX_APD_AUTOSET_AMPL       134        // APD Auto Gain Target Signal Level
#define NEOFOX_APD_DAC_MAX            135        // APD voltage DAC value for max setting
#define NEOFOX_APD_DAC_145V           136        // APD voltage DAC value for 145V
#define NEOFOX_LED_CURRENT_DAC_MAX    137        // DAC value for maximum LED current
#define NEOFOX_LED_RECOVERY_CYCLES    138        // # of LED cycles to ignore after LED blue/red toggle
#define NEOFOX_DISABLE_RED_LED        139        // Disable use of red LED in algorithm processing

#define NEOFOX_DAC_LED_CURRENT_RED    140        // Red LED current level
#define NEOFOX_DAC_APD_GAIN           141        // Avalanche photodiode gain (voltage)
#define NEOFOX_DAC_OFFSET_ADJUST      142        // AFE baseline adjust
#define NEOFOX_DAC_LED_CURRENT_BLUE   143        // Blue LED current level
#define NEOFOX_DAC_LED_CURRENT_OFF    144        // Off LED current level
#define NEOFOX_RESERVED_145           145        
#define NEOFOX_RESERVED_146           146        
#define NEOFOX_RESERVED_147           147        
#define NEOFOX_BOARD_TEMP_ENA         148        // Board temperature readings
#define NEOFOX_GET_ONLY_LED_LO        149        // Option to retreive only low parts of the waveform
#define NEOFOX_APD_DAC_OPTIMAL        150        // Best DAC setting for APD noise / gain tradeoff
#define NEOFOX_PHASE_AVG_CNT_RED      151        // # phase results to average for red LED
#define NEOFOX_OXYGEN_UNITS           152        // Units of measurement for oxygen

// Sensor Calibration
#define NEOFOX_CAL_NOTES              160        // Calibration notes
#define NEOFOX_CAL_SERIAL_NUMBER      161        // Sensor Serial Number
#define NEOFOX_CAL_SENSOR_TYPE        162        // Sensor type
#define NEOFOX_CAL_METHOD             163        // Type of calibration to use
#define NEOFOX_CAL_DEFAULT_TEMP       164        // Default temperature to use if no sensor
#define NEOFOX_CAL_TEMP_SOURCE        165        // Temperature source to use when computing O2
#define NEOFOX_CAL_DATE               166        // Calibration date

#define NEOFOX_CAL_2PT_TAU_0          170        // Two point calibration data point
#define NEOFOX_CAL_2PT_TAU_1          171        // Two point calibration data point
#define NEOFOX_CAL_2PT_O2_0           172        // Two point calibration data point
#define NEOFOX_CAL_2PT_O2_1           173        // Two point calibration data point
#define NEOFOX_CAL_2PT_SLOPE          174        // Slope - Two point calibration equation 
#define NEOFOX_CAL_2PT_OFFSET         175        // Offset - Two point calibration equation

// Calibration
#define NEOFOX_CALIBRATE_TEMP0        180        // Calibration constants
#define NEOFOX_CALIBRATE_TEMP1        181        
#define NEOFOX_CALIBRATE_TEMP2        182        
#define NEOFOX_CALIBRATE_TEMP3        183        

#define NEOFOX_CALIBRATE_PRESSURE     185        

// Single point cal
#define NEOFOX_CAL_1PT_TAU            186        // Single point update for mutli point cal
#define NEOFOX_CAL_1PT_O2             187
#define NEOFOX_CAL_1PT_TEMPERATURE    188
#define NEOFOX_CAL_1PT_CALCULATE      189        // Perform the single point calibration

// Multipoint cal
#define NEOFOX_CAL_MULTI_A0           200        // Calibration constants
#define NEOFOX_CAL_MULTI_A1           201
#define NEOFOX_CAL_MULTI_A2           202
#define NEOFOX_CAL_MULTI_B0           203
#define NEOFOX_CAL_MULTI_B1           204
#define NEOFOX_CAL_MULTI_B2           205
#define NEOFOX_CAL_MULTI_C0           206
#define NEOFOX_CAL_MULTI_C1           207
#define NEOFOX_CAL_MULTI_C2           208
#define NEOFOX_CAL_MULTI_T0           209           
#define NEOFOX_CAL_MULTI_T1           210           
#define NEOFOX_CAL_MULTI_T2           211

#define NEOFOX_AOUT_VOLTAGE_SOURCE    212  // Selects the data source for the analog voltage output
#define NEOFOX_AOUT_CURRENT_SOURCE    213  // Selects the data source for the analog 4-20 mA output
#define NEOFOX_AOUT_VOLTAGE_LBOUND    214  // Lower bound for the analog voltage output
#define NEOFOX_AOUT_VOLTAGE_UBOUND    215  // Upper bound for the analog boltage output
#define NEOFOX_AOUT_CURRENT_LBOUND    216  // Lower bound for the analog 4-20 mA output
#define NEOFOX_AOUT_CURRENT_UBOUND    217  // Upper bound for the analog 4-20 mA output

// Download and Update
#define NEOFOX_UPDATE_FIRMWARE        220        // Firmware update, must pass valid checksum

#define NEOFOX_DOWNLOAD_CHKSUM        224        // Checksum of downloaded data
#define NEOFOX_DOWNLOAD_FRAMES        225        // Number of download frames received
#define NEOFOX_DOWNLOAD_FRAMES_DROP   226        // Number of download frames dropped
#define NEOFOX_DOWNLOAD_ERRORS        227        // Number of programming errors

// For host use only
#define HOST_DROPOUTS                 231        // Number of dropped frames
#define HOST_PHASE_PC                 232        // PC calculated phase
#define HOST_EXPORT_INTERVAL          233        // Data file export interval
#define HOST_STDDEV_INTERVAL          234        // Standard deviation length in seconds
#define HOST_BLUE_PHASE_STDDEV        235        // Phase standard deviation in 1000th's of a degree
#define HOST_RED_PHASE_STDDEV         236        // Phase standard deviation in 1000th's of a degree
#define HOST_TAU_CURVE                237        // Exponential curve fit Tau
#define HOST_TAU_PHASE_STDDEV         238        // Tau (phase method) standard deviation in nsec
#define HOST_TAU                      239        // Host calculate Tau
#define HOST_TAU_PRECISION            240        // Tau Precision (Tau Std Dev/Tau Phase)
#define HOST_CONNECTION_STATUS        241        // State of connection

//-------------------------------------------------------------------------
// Misc. Definitions
//-------------------------------------------------------------------------
#define SENSOR_ARRAYSIZE           1024        // Number of data samples in NEOFOX frame
#define FFT_SIZE                   1024
#define SENSOR_ARRAYSIZE_SHIFT       10        // Number for averaging samples (2 Nth power)

#define TEMPERATURE_ARRAYSIZE         3        // Number of temperature fields

// Diagnostic LED States
#define DIAG_LED_OFF                  0
#define DIAG_LED_ON                   1
#define DIAG_LED_BLINK                2        // Blink approximately once per second

// AutoGain state tracking
#define AUTOGAIN_STATE_READY                    0

#define AUTOGAIN_STATE_SETUP_RED_APD            10
#define AUTOGAIN_STATE_RED_APD                  11
#define AUTOGAIN_STATE_RED_APD_PHASE2           12

#define AUTOGAIN_STATE_SETUP_TEST_CYCLE         20
#define AUTOGAIN_STATE_TEST_CYCLE               21
#define AUTOGAIN_STATE_TEST_CYCLE_FINAL         22

#define AUTOGAIN_STATE_SETUP_FINE_TUNING_RED    30
#define AUTOGAIN_STATE_FINAL_FINE_TUNING_RED    31

#define AUTOGAIN_STATE_START                    AUTOGAIN_STATE_SETUP_RED_APD

#define AUTOGAIN_STATE_ABORT                    99
  
// Type of analog data being sent
#define ANALOG_DATATYPE_SENSOR        0
#define ANALOG_DATATYPE_RED_LED       1

// Simulated waveforms
#define SENSOR_DTYPE_SENSOR           0
#define SENSOR_DTYPE_SINE0            1
#define SENSOR_DTYPE_SINE90           2
#define SENSOR_DTYPE_LEDSQUARE        3

#define NEOFOX_AOUT_SRC_OFF           0
#define NEOFOX_AOUT_SRC_O2            1
#define NEOFOX_AOUT_SRC_TEMP          2
#define NEOFOX_AOUT_SRC_PRESSURE      3
#define NEOFOX_AOUT_SRC_TAU           4
#define NEOFOX_AOUT_SRC_O2_CONV       5

#define NEOFOX_O2_UNITS_PERCENT       0      //  %
#define NEOFOX_O2_UNITS_DO_PPM        1      //  DO - ppm - Water     
#define NEOFOX_O2_UNITS_HPA           2      //  hPa - Partial Pressure
#define NEOFOX_O2_UNITS_MBAR          3      //  mbar - Partial Pressure
#define NEOFOX_O2_UNITS_TORR          4      //  Torr - Partial Pressure
#define NEOFOX_O2_UNITS_MG_L          5      //  mg/L - Concentration 
#define NEOFOX_O2_UNITS_PPM           6      //  ppm - Concentration  
#define NEOFOX_O2_UNITS_UMOL_L        7      //  umol/L - Concentration  

//--------------------------------------------------------
// ADC scaling, oversampling, normalize to 16 bits
//--------------------------------------------------------
#define ADC10_SCALER                 64
#define ADC10_SHIFT                   6

//--------------------------------------------------------
// Misc. Definitions
//--------------------------------------------------------
#define SNR_SHIFT                     4        // Scaling Factor (Left Right Shift)
#define SNR_SCALER                   16        //

#define MAX_PERCENTAGE_VALUE       1000        // Tenths of a percent

// Generic data streams
#define UART_BUFFER_SIZE           1100        // Max size of NEOFOX host cmd frame

#define PROTOCOL_ETX               0x04        // Header
#define PROTOCOL_STX               0x03        // Footer

// Some NEOFOX Maximums
#define NEOFOX_MIN_VALUE              0        // Minimum NEOFOX Analog Data Value
#define NEOFOX_MAX_VALUE          65535        // Maximum NEOFOX Analog Data Value
#define NEOFOX_DAC_MAX_VALUE      65535        // Maximum NEOFOX DAC Value

                                               // In 0.1 second increments, support up to 1 minute averaging
#define PHASE_AVG_CNT_MAX           300        // Maximum number of phase, O2, temp, pressure, board voltages, Vref values to average
#define AUX_AVG_CNT_MIN             100        // Minimum averaging for red LED, APD voltage, and other non phase circuits

#define AVERAGE_SIZE_8                8        // Maximum number of integer values to average
#define MAX_LCD_TEXT                100        // Maximum character in text command

#define LCD_SIZE_X                  128        // LCD Size in pixels
#define LCD_SIZE_Y                   64
                                               // Right align of half screen size 
#define LCD_HALF_RIGHT_X           ((LCD_SIZE_X/2)-1)        

#define NEOFOX_TEXT_DISABLE           255        // Disable text display

#define NEOFOX_FONT_STD                 0        // Supported Fonts
#define NEOFOX_FONT_BOLD                1
#define NEOFOX_FONT_ARIEL18             2
#define NEOFOX_FONT_ARIEL9              3
#define NEOFOX_FONT_CP8859_14           4
#define NEOFOX_FONT_CP8859_9            5
#define NEOFOX_FONT_MONO5_8             6
#define NEOFOX_FONT_MONO8_8             7
#define NEOFOX_FONT_MSFONT              8
#define NEOFOX_FONT_MSFONT78            9
#define NEOFOX_FONT_NARROW10           10
#define NEOFOX_FONT_TIMES13            11
#define NEOFOX_FONT_TIMES16            12
#define NEOFOX_FONT_TIMES9             13

#define MAX_DATE_STRING                16        // Maximum characters in a string
#define MAX_PARAM_STRING               32        // Maximum characters in a string
#define MAX_CAL_STRING                 64        // Maximum string in calibration notes
#define MAX_STRING                     64        // Maximum string passed from host
#define MAX_DOWNLOAD_SIZE           (1024)       // Amount of download data in one frame 

#define SERIAL_LENGTH                 8        // 8 byte serial number

// Sensor Autoset
#define NEOFOX_AUTOSET_FOXY             0 
#define NEOFOX_AUTOSET_HIOXY            1 
#define NEOFOX_AUTOSET_FOSPOR           2 

// Temperature Source
#define NEOFOX_TEMP_SOURCE_NONE         0
#define NEOFOX_TEMP_SOURCE_SENSOR       1
#define NEOFOX_TEMP_SOURCE_FIXED        2

// Display Modes
#define NEOFOX_DM_RESET                 0        // Application reset
#define NEOFOX_DM_LOGO                  1        // Logo display
#define NEOFOX_DM_NEOFOX                2        // NEOFOX display
#define NEOFOX_DM_CONNECT               3        // Connect PC Display
#define NEOFOX_DM_IDLE                  4        // Waiting for a run start
#define NEOFOX_DM_ACTIVE                5        // Run time status display
#define NEOFOX_DM_HOST                  6        // Host Controlled Mode
#define NEOFOX_DM_TILT                  7        // Tilt Grid

//------------------------------------------------------------------
// Misc Defintions For Hardware Settings
//------------------------------------------------------------------
#define NEOFOX_DAC_ADDR_APD_GAIN         0   // Avalanche photodiode gain (voltage)
#define NEOFOX_DAC_ADDR_LED_CURRENT_ON   1   // LED current during off cycle");
#define NEOFOX_DAC_ADDR_LED_CURRENT_OF   2   // LED current during on cycle");
#define NEOFOX_DAC_ADDR_OFFSET_ADJUST    3   // AFE baseline adjust");

#define NEOFOX_PGA_GAIN_PATH_LO     0   // AFE Amplifier gain path
#define NEOFOX_PGA_GAIN_PATH_HI     1
                                        // AFE Amplifier gain level
#define NEOFOX_PGA_GAIN_1x          0   // LoGain = 4, HiGain = 0, Baseline 1.26x
#define NEOFOX_PGA_GAIN_2x          1   // LoGain = 5, HiGain = 0, 
#define NEOFOX_PGA_GAIN_4x          2   // LoGain = 6, HiGain = 0, 
#define NEOFOX_PGA_GAIN_8x          3   // LoGain = 7, HiGain = 0, 
#define NEOFOX_PGA_GAIN_20x         4   // LoGain = 5, HiGain = 4, 2nd stage = 10x 
#define NEOFOX_PGA_GAIN_40x         5   // LoGain = 5, HiGain = 5, 2nd stage = 10x 
#define NEOFOX_PGA_GAIN_80x         6   // LoGain = 5, HiGain = 6, 2nd stage = 10x 
#define NEOFOX_PGA_GAIN_160x        7   // LoGain = 5, HiGain = 7, 2nd stage = 10x 
        
#define NEOFOX_PGA_HI_GAIN_DEFAULT  5   // Setting for lo gain stage when using hi gain page

#define NEOFOX_LED_BLUE             0   // ID's for selecting LED data
#define NEOFOX_LED_RED              1
#define NEOFOX_NUM_LEDS             2

#define NEOFOX_LED_CTL_OFF          0   // Led Control Register
#define NEOFOX_LED_CTL_BLUE_LED     1
#define NEOFOX_LED_CTL_RED_LED      2
#define NEOFOX_LED_CTL_TOGGLE_LED   3
#define NEOFOX_LED_CTL_DAC_ENA_N    4

#define NEOFOX_LED_REC_TIME_2       0   // LED Record Time 
#define NEOFOX_LED_REC_TIME_4       1    
#define NEOFOX_LED_REC_TIME_8       2    
#define NEOFOX_LED_REC_TIME_16      3    
#define NEOFOX_LED_REC_TIME_32      4    
#define NEOFOX_LED_REC_TIME_64      5    
#define NEOFOX_LED_REC_TIME_128     6    
#define NEOFOX_LED_REC_TIME_256     7    
#define NEOFOX_LED_REC_TIME_512     8    
#define NEOFOX_LED_REC_TIME_1024    9    
#define NEOFOX_LED_REC_TIME_2048   10    

// Basic impulse waveform phase offset for base waveforms
// 2 edge only pulses per waveform
#define NEOFOX_PULSE_PHASE_OFFSET_2X  (-1.5707963267948966)

// We only allow even numbers (N+1) here due to hardware limitation
// Setting for clk divider (LED Timing = 128/256)
#define NEOFOX_LED_RATE_46_88_KHZ  (2 * 0)
#define NEOFOX_LED_RATE_23_44_KHZ  (2 * 1)
#define NEOFOX_LED_RATE_11_72_KHZ  (2 * 3)
#define NEOFOX_LED_RATE_7_813_KHz  (2 * 5)
#define NEOFOX_LED_RATE_5_859_KHZ  (2 * 7)
#define NEOFOX_LED_RATE_4_688_KHZ  (2 * 9)
#define NEOFOX_LED_RATE_2_344_KHZ  (2 * 19)
#define NEOFOX_LED_RATE_1_563_KHZ  (2 * 29)
#define NEOFOX_LED_RATE_1_172_KHZ  (2 * 39)
#define NEOFOX_LED_RATE_0_938_KHZ  (2 * 49)
#define NEOFOX_LED_RATE_0_469_KHZ  (2 * 99)
#define NEOFOX_LED_RATE_0_234_KHZ  (2 * 199)

// Supported auto configure clock rates
#define LED_CLK_93750              93750
#define LED_CLK_46875              46875
#define LED_CLK_23438              23438
#define LED_CLK_11719              11719
#define LED_CLK_5859               5859
#define LED_CLK_2930               2930
#define LED_CLK_1465               1465
#define LED_CLK_977                977
#define LED_CLK_732                732

// Scaler for Phase to Tau
#define RADIANS_TO_DEGREES_SCALER   ((float) (180.0 / X_PI))

// Scaler for radians to Tau
#define PHASE_TAU_SCALER_93750      ((float) (((1000000.0 / 93750.0)   / 360.0) * RADIANS_TO_DEGREES_SCALER))
#define PHASE_TAU_SCALER_46875      ((float) (((1000000.0 / 46875.0)   / 360.0) * RADIANS_TO_DEGREES_SCALER))
#define PHASE_TAU_SCALER_23438      ((float) (((1000000.0 / 23437.5)   / 360.0) * RADIANS_TO_DEGREES_SCALER))
#define PHASE_TAU_SCALER_11719      ((float) (((1000000.0 / 11718.75)  / 360.0) * RADIANS_TO_DEGREES_SCALER))
#define PHASE_TAU_SCALER_5859       ((float) (((1000000.0 / 5859.375)  / 360.0) * RADIANS_TO_DEGREES_SCALER))
#define PHASE_TAU_SCALER_2930       ((float) (((1000000.0 / 2929.6875) / 360.0) * RADIANS_TO_DEGREES_SCALER))
#define PHASE_TAU_SCALER_1465       ((float) (((1000000.0 / 1464.84375)/ 360.0) * RADIANS_TO_DEGREES_SCALER))
#define PHASE_TAU_SCALER_977        ((float) (((1000000.0 / 976.5625)  / 360.0) * RADIANS_TO_DEGREES_SCALER))
#define PHASE_TAU_SCALER_732        ((float) (((1000000.0 / 732.421875)/ 360.0) * RADIANS_TO_DEGREES_SCALER))

#define NEOFOX_STATUS_FIFO_CPU_PROG_FULL    0x0001 
#define NEOFOX_STATUS_FIFO_CPU_PROG_EMPTY   0x0002 
#define NEOFOX_STATUS_FIFO_CPU_AVAILABLE    0x0004 
#define NEOFOX_STATUS_DAC_WRITE_COMPLETE    0x0008 
#define NEOFOX_STATUS_VERSION               0x00F0 
#define NEOFOX_STATUS_RESET                 0x0800 
#define NEOFOX_STATUS_FIFO_CPU_EMPTY        0x1000 
#define NEOFOX_STATUS_FIFO_CPU_FULL         0x2000 
#define NEOFOX_STATUS_FIFO_SENSOR_EMPTY     0x4000 
#define NEOFOX_STATUS_FIFO_SENSOR_FULL      0x8000 

#define NEOFOX_BOARD_TEMP_INVALID           0x3FFFFFFF   // Board temperatures are not valid

#define NEOFOX_CAL_METHOD_NONE              0            // Calibration methods
#define NEOFOX_CAL_METHOD_TWO_POINT         1
#define NEOFOX_CAL_METHOD_MULTI_POINT       2
#define NEOFOX_CAL_METHOD_SINGLE_POINT      3

//------------------------------------------------------------------
// The host should wait approx. 10 msec before sending a frame in
//   the same category here
//------------------------------------------------------------------
#define NEOFOX_CMD_SET_PARAM_PC      200       // Host Is Setting A Parameter On NEOFOX
#define NEOFOX_CMD_SET_PARAM_TEXT    204       // Host Is Setting A Parameter String

//------------------------------------------------------------------
// These commands are rcv / xmt only once
//------------------------------------------------------------------
#define NEOFOX_CMD_DATA              220       // NEOFOX Analog Data
#define NEOFOX_CMD_DOWNLOAD          224       // Download frame

//---------------------------------------------
// Serial Port Protocol Structures
//
// WARNING: Do not use int! This file is
//          processed by both 16 and 32 bit
//          compilers,use only short or long
//---------------------------------------------
#define NEOFOX_PROTOCOL_REV          1

//------------------------------------------------------------------
// Force Alignment On Serial Transfer Data For XMT / RCV
//------------------------------------------------------------------

//----------------------------------------------------------------
// Custom Strings For NEOFOX Display
//----------------------------------------------------------------
struct CustomStringPos
{
  unsigned char   X;
  unsigned char   Y;
  unsigned char   Flags;
  unsigned char   Font;
};

//----------------------------------------------------------------
// NEOFOX Calibration Data
//----------------------------------------------------------------
struct NeoFoxCalSinglePoint_Data
{
  float           fTau;                   // Most recent single point cal data
  float           fO2;
  float           fTemperature;
};

struct NeoFoxCalMultiPoint_Data
{
  float           fA_0;                   // Calibration constants
  float           fA_1;
  float           fA_2;

  float           fB_0;                   
  float           fB_1;
  float           fB_2;

  float           fC_0;                   
  float           fC_1;
  float           fC_2;

  float           fT_0;                   
  float           fT_1;
  float           fT_2;
};

struct NeoFoxCalTwoPoint_Data
{
  float           fTau_0;                 // Data points used
  float           fTau_1;                
  float           fO2_0;                 
  float           fO2_1;                 
    
  float           fSlope;                 // Linear Equation 
  float           fOffset;                 
  
  float           fTemperature;           // Temperature used during cal       
};

struct NeoFoxCalibration_Data
{
  struct NeoFoxCalSinglePoint_Data  SinglePoint;       // Single point calibration of MultiPoint equation
  struct NeoFoxCalTwoPoint_Data     TwoPoint;          // Two point calibration / linear

  struct NeoFoxCalMultiPoint_Data   MultiPointOrig;    // The original constants for multiPoint cal (before single point adjustment)
  struct NeoFoxCalMultiPoint_Data   MultiPoint;        // MultiPoint calibration / polynomial

  float           fDefaultTemp;                     // Default temperature to use if no sensor

  unsigned long   uiMethod;                         // Type of calibration to use
  unsigned long   uiSensorType;                     // Type of sensor this calibration is for
  unsigned long   uiTempSource;                     // Temperature source to use when computing O2 
  
  char            Notes[MAX_CAL_STRING];            // Calibration notes
  char            SerialNumber[MAX_PARAM_STRING];   // Sensor Serial Number
  char            Date[MAX_DATE_STRING];            // Calibration date

  unsigned char   Reserved[32];                     // Reserved for future expansion
};

//----------------------------------------------------------------
// Non-volatile parameters stored to flash
//----------------------------------------------------------------
// NEOFOX Parameters
struct Flash_Data
{
  unsigned int    Init;
  unsigned int    Revision;                       // Storage revision
                                                  // Temperature and pressure calibration
  int             iCalibrateTemp[TEMPERATURE_ARRAYSIZE+1];
  int             iCalibratePressure;
    
  unsigned int    uiApdAutosetAmplitude;          // Auto Gain APD target gain amplitude
  
  unsigned int    uiApdDacMax;                    // APD voltage DAC value for maximum allowed setting
  unsigned int    uiApdDacOptimal;                // APD voltage DAC value for best gain / noise setting
  unsigned int    uiApdDac145V;                   // APD voltage DAC value for 145V
  unsigned int    uiLedCurrentDacMax;             // DAC value for maximum LED current
  unsigned int    uiSensorAutoSet;                // Set default settings based on sensor type
  unsigned int    uiDutyCycleSecOn;               // LED duty cycling, number of seconds to stay on
  unsigned int    uiDutyCycleSecOff;              // LED duty cycling, number of seconds to remain off
  
  unsigned long   uiPhaseAvgCntBlue;              // # phase results to average
  unsigned long   uiPhaseAvgCntRed;               // # phase results to average
  unsigned long   uiDiagnosticsLedOn;             // Main Diagnostics LED Status
  unsigned long   uiWriteJaz;                     // Write the Jaz interface

  char            SerialNumber[MAX_PARAM_STRING]; // NEOFOX Serial Number
  char            Name[MAX_PARAM_STRING];         // NEOFOX Name
  
  struct NeoFoxCalibration_Data Calibration;      // Calibration data

  unsigned long   uiEnableLCD;                    // Enable the LCD Controller

  char            chAnalogVoltageOutputSource;    // Selects the data source for the analog voltage output //[AOUT]
  char            chAnalogCurrentOutputSource;    // Selects the data source for the analog 4-20 mA output
  char            Reserved2;  
  char            Reserved3;  

  float           fAnalogVoltageLBound;           // Lower bound for the analog voltage output
  float           fAnalogVoltageUBound;           // Upper bound for the analog boltage output
  float           fAnalogCurrentLBound;           // Lower bound for the analog 4-20 mA output
  float           fAnalogCurrentUBound;           // Upper bound for the analog 4-20 mA output
  
  unsigned long   uiOxygenUnits;                  // Units of measurement for O2 

  unsigned char   Reserved[6];                    // Reserved for future expansion  //[AOUT]
};

//----------------------------------------------------------------
// NEOFOX Per LED Settings
//----------------------------------------------------------------
struct NeoFoxLedSettings_Data
{
  unsigned long   uiPgaGain;             // Programmable gain amplifier setting for LED
  unsigned long   uiDacCurrent;          // LED current
  unsigned long   uiPgaGain1stStage;     // Programmable gain setting of first stage when in in high gain path
};

//----------------------------------------------------------------
// NEOFOX Parameters
//----------------------------------------------------------------
struct NeoFoxSettings_Data
{
  // Settings
  struct NeoFoxLedSettings_Data  RedLed;       // Led specific settings
  struct NeoFoxLedSettings_Data  BlueLed;
  
  unsigned long   uiLedSequenceOn;             // Control LED sequencing on / off
  unsigned long   uiLedControl;                // Misc. LED control 
  unsigned long   uiAdcControl;                // Misc. ADC control = AdcOEn,  [Gain * 1.5],  Rand, Mode (0/1), Dither,  Shutdown
  unsigned long   uiAdcClkDivider;             // Divisor for main clock to create system clock
  unsigned long   uiLedOnTime;                 // # of clocks to hold LED high 
  unsigned long   uiLedCycleTime;              // # of clocks for entire LED cycle 
  unsigned long   uiLedRecTime;                // # of ADC samples to record (power of 2 encoded)
  unsigned long   uiLedAvgCycles;              // # LED cycles to average 
  unsigned long   uiLedRecoveryCycles;         // # LED cycles to ignore after red / blue toggle change
  unsigned long   uiSensorDataType;            // Simulated waveforms
  float           fBaselinePhase;              // Set baseline phase (subtracted fronm computed phase)
  unsigned long   uiLedClockRate;              // Set clock rate, optimize settings

  unsigned long   uiDacApdGain;                // Avalanche photodiode gain (voltage)
  unsigned long   uiDacLedCurrentOff;          // Off LED current
  unsigned long   uiDacOffsetAdjust;           // AFE baseline adjust

  unsigned int    uiAutoGainLevelLo;           // Lowest allowed level before autogain occurs
  unsigned int    uiAutoGainLevelHi;           // Highest allowed level before autogain occurs
  unsigned int    uiAutoGainAmplitudeLo;       // Lowest amplitude allowed before autogain occurs
  unsigned int    uiAutoGainAmplitudeHi;       // Highest amplitude allowed before autogain occurs
  unsigned int    uiAutoGainEnable;            // Automatic gain sequence for sensor enable
  
  unsigned long   uiAnalogOut_4_20_ma;         // Analog out - current
  unsigned long   uiAnalogOut5V;               // Analog out - voltage
  unsigned long   uiBoardTempEna;              // Board temperature reading enabled
  unsigned long   uiDisableRedLed;             // Disable use of red LED in algorithm calculations
  unsigned long   uiGetOnlyLedLo;              // Only use falling edge part of waveform (not supported)
  
  unsigned char   Reserved[32];                // Reserved for future expansion
};

//----------------------------------------------------------------
// NEOFOX Data That Stored For Each LED
//----------------------------------------------------------------
struct NeoFoxLED_Data
{
  float         fPhase;                        // Final phase calculation
  float         fPhaseRaw;                     // Most recent phase calculation
  unsigned int  uiSNR;                         // Waveform signal quality
  unsigned int  uiDecayTime;                   // Waveform decay time
  unsigned int  uiPeakLo;                      // Waveform amplitude low
  unsigned int  uiPeakHi;                      // Waveform amplitude high
  unsigned int  uiAvgPeakLo;                   // Average peak in data
  unsigned int  uiAvgPeakHi;                   
  unsigned int  uiAvgAmplLo;                   // Average amplitude before rising edge
  unsigned int  uiAvgAmplHi;                   
};

// For file downloaded 
struct NeoFoxDownload
{
  // Download 
  BYTE *bBuffer;                      // Pointer to download area
  int  TotalSizeBytes;                // Total number of bytes indownload
  int  TotalFrames;                   // Total frames in download
  int  MostRecentFrameNum;            // Number of download frames received
  int  DroppedFrames;                 // Number of download frames dropped
  int  ByteCheckSum;                  // Checksum of downloaded data
  int  Errors;                        // Number of programming errors
};                

//----------------------------------------------------------------
// NEOFOX Status
//----------------------------------------------------------------
struct NeoFoxStatus_Data
{
  struct NeoFoxLED_Data Blue;                 // Data for each LED
  struct NeoFoxLED_Data Red;

  float         fTau;                         // Calculated TAU
  float         fOxygen;                      // Calculated O2 level
  
  unsigned int  uiBoardTemperature1;          // Board Temperature Readings
  unsigned int  uiBoardTemperature2;         
  unsigned int  uiBoardTemperature3;         

  unsigned int  uiRawAdcPressure;              // Raw ADC readings
  unsigned int  uiRawAdcBoardVoltage;          
  unsigned int  uiRawAdcBatteryVoltage;          
  unsigned int  uiRawAdcApdVoltage;            
  unsigned int  uiRawAdcTemperature;           
  unsigned int  uiRawAdcVrefSensor;        
                                               // Translated readings to proper units for parameter 
  unsigned int  uiAdcPressure;                 // Air pressure reading
  unsigned int  uiAdcBoardVoltage;             // Measured 5.5V power supply level
  unsigned int  uiAdcBatteryVoltage;           // Measured battery power supply level
  unsigned int  uiAdcApdVoltage;               // Measured power supply level of APD supply
  int           iAdcTemperature;               // Temperature sensor reading
  unsigned int  uiAdcVrefSensor;               // Voltage reference for sensor measurement
  
  unsigned int  uiFpgaStatus;                  // Status register of FPGA
  unsigned int  uiDisplayMode;                 // Display state of the NEOFOX
                                               // Board Serial Number from external chip
  unsigned char SerialNumber[SERIAL_LENGTH];  
                
  unsigned int  uiApdAutosetState;             // APD Auto Gain Status

  unsigned int  uiRunStartTime;                // Start time of last run
  unsigned int  uiRunStopTime;                 // End time of last run
  
  unsigned int  uiPostResults;                 // Power On Self Test Results

  struct NeoFoxDownload  Download;             // Download tracking

  float         fOxygenConverted;              // Optional Alternate O2 Calculation for display

  unsigned int  uiDutyCycleSecCnt;             // Incrementing second count for duty cycling
  unsigned int  uiDutyCycleSecTick;            // Flag that we just hit a second boundary
  unsigned int  uiDutyCycleLedState;           // On / Off state of duty cycle
  
  unsigned char ucReserved[16];                // Reserved for future expansion  //[DisO2]
};

//----------------------------------------------------------------
// NEOFOX Raw Sensor Data For Diagnostics
//----------------------------------------------------------------
struct NeoFoxSensor_Data
{
  unsigned int   uiNumSamples;                 // Number of entries
  unsigned short usData[SENSOR_ARRAYSIZE];     // ADC sensor data
};

//----------------------------------------------------------------
// NEOFOX Data - Transferred to host
//----------------------------------------------------------------
struct NeoFoxParams_Data
{
  // Frame Specific Data
  long                        iSize;              // Size of this structure

  unsigned char               ucVersionHi;        // Software Revisions
  unsigned char               ucVersionLo;
  unsigned short              usHostCmdDrop;      // Number of dropped graphics commands

  unsigned long               uiHostRxBytesAvail; // Bytes available for RX
  volatile unsigned long      uiMillisecondCount; // Millisecond counter since processor reset

  unsigned long               uiFlashWriteCnt;    // Number of flash memory updates

  struct Flash_Data           Flash;              // RAM copy of non-volatile flash memory storage
  struct NeoFoxSettings_Data  Settings;           // Board setup
  struct NeoFoxStatus_Data    Status;             // Board status

  unsigned char               Reserved[32];       // Reserved for future expansion
};

//------------------------------------------------------------------
// Host Parameter Set
//   NOTE: All Host To NEOFOX structures must have 4 char's followed
//         by the 4 byte command number
//------------------------------------------------------------------
struct ProtNeoFoxSetParam
{
  // Header
  unsigned char     Stx;
  unsigned char     PacketType;
  unsigned short    FrameSize;

  // Frame Specific Data
  unsigned long     CmdNumber;   // Host Command Number
  unsigned long     ParamType;
  long              ParamValue;

  // Trailer
  unsigned char     Reserved1;   // Byte padding for word alignment
  unsigned char     Reserved2;   // Byte padding for word alignment
  unsigned char     Checksum;
  unsigned char     Eof;
};

//------------------------------------------------------------------
// Host Parameter Set String
//   NOTE: All Host To NEOFOX structures must have 4 char's followed
//         by the 4 byte command number
//------------------------------------------------------------------
struct ProtNeoFoxSetParamString
{
  // Header
  unsigned char     Stx;
  unsigned char     PacketType;
  unsigned short    FrameSize;

  // Frame Specific Data
  unsigned long     ParamType;
  char              ParamString[MAX_STRING];

  // Trailer
  unsigned char     Reserved1;   // Byte padding for word alignment
  unsigned char     Reserved2;   // Byte padding for word alignment
  unsigned char     Checksum;
  unsigned char     Eof;
};

//------------------------------------------------------------------
// Host LCD Graphics Clear Command
//   NOTE: All Host To NEOFOX structures must have 4 char's followed
//         by the 4 byte command number
//------------------------------------------------------------------
struct ProtNeoFoxClrGraphic
{
  // Header
  unsigned char     Stx;
  unsigned char     PacketType;
  unsigned short    FrameSize;

  // Frame Specific Data
  unsigned long     CmdNumber;    // Host Command Number
  short             PositionX;    // Size and Position To Clear
  short             PositionY;
  short             SizeX;
  short             SizeY;

  // Trailer
  unsigned char     Reserved1;   // Byte padding for word alignment
  unsigned char     Reserved2;   // Byte padding for word alignment
  unsigned char     Checksum;
  unsigned char     Eof;
};

//------------------------------------------------------------------
// Host LCD Graphics Text Command
//   NOTE: All Host To NEOFOX structures must have 4 char's followed
//         by the 4 byte command number
//------------------------------------------------------------------
struct ProtNeoFoxText
{
  // Header
  unsigned char     Stx;
  unsigned char     PacketType;
  unsigned short    FrameSize;

  // Frame Specific Data
  unsigned char     TextFlags;    // Alignment, etc.
  unsigned char     FontType;     // Font To Use
  unsigned char     TextReserved2;
  unsigned char     TextReserved3;

  unsigned long     CmdNumber;    // Host Command Number
  short             PositionX;    // Size and Position To Clear
  short             PositionY;

  // Text String
  char              szString[MAX_LCD_TEXT];

  // Trailer
  unsigned char     Reserved1;   // Byte padding for word alignment
  unsigned char     Reserved2;
  unsigned char     Checksum;
  unsigned char     Eof;
};

//------------------------------------------------------------------
// Host LCD Graphics Command
//   NOTE: All Host To NEOFOX structures must have 4 char's followed
//         by the 4 byte command number
//------------------------------------------------------------------
struct ProtNeoFoxSetGraphic
{
  // Header
  unsigned char     Stx;
  unsigned char     PacketType;
  unsigned short    FrameSize;

  // Frame Specific Data
  unsigned long     CmdNumber;    // Host Command Number
  short             PositionX;    // Size and Position of Graphic
  short             PositionY;
                                  // The 4 items below match the native header of the LCD library
  unsigned char     colorid;      // Color header id (= B&W_x = 0) */
  unsigned char     pbits;        // Number of bits pr pixel (= color mode) */
  unsigned char     cxpix;        // Symbol size in no X pixels */
  unsigned char     cypix;        // Symbol size in no Y pixels */

  // Bitmap data - (cxpix * cypix) * pbits (4 Bits per Pixel)
  //  The graphics data will be inserted here and should
  //  be padded to be on a 4 byte boundary
  unsigned char     gData[LCD_SIZE_X/2];    // Place holder

  // Trailer
  unsigned char     Reserved1;   // Byte padding for word alignment
  unsigned char     Reserved2;   // Byte padding for word alignment
  unsigned char     Checksum;
  unsigned char     Eof;
};

//------------------------------------------------------------------
// Revision #2 8-06
//------------------------------------------------------------------
//
//  This protocol receives dynamic length frames that is intended
//    to send a variety of commands.
//
//------------------------------------------------------------------
// NEOFOX Data Frame To Host
//------------------------------------------------------------------
struct ProtNeoFoxHostData
{
  // Header
  unsigned char         Stx;
  unsigned char         PacketType;
  unsigned short        FrameSize;

  // Frame specific information
  unsigned char         FrameCount;        // Incrementing frame counter
  unsigned char         ProtocolRev;       // Revision of this protocol
  unsigned char         ParamReserved2;    // Byte padding for word alignment
  unsigned char         ParamReserved3;    // Byte padding for word alignment

  struct NeoFoxParams_Data  NeoFoxParams;      // NeoFox Parameters
  struct NeoFoxSensor_Data  NeoFoxSensorBlue;  // NeoFox Waveform Sensor Data
  struct NeoFoxSensor_Data  NeoFoxSensorRed; 

  // Trailer
  unsigned char         Reserved3;         // Byte padding for word alignment
  unsigned char         Reserved4;         // Byte padding for word alignment

  unsigned char         Checksum;          // Checksum of data
  unsigned char         Eof;               // End of frame marker
};

struct ProtNeoFoxHostData_NoWaveforms
{
  // Header
  unsigned char         Stx;
  unsigned char         PacketType;
  unsigned short        FrameSize;

  // Frame specific information
  unsigned char         FrameCount;        // Incrementing frame counter
  unsigned char         ProtocolRev;       // Revision of this protocol
  unsigned char         ParamReserved2;    // Byte padding for word alignment
  unsigned char         ParamReserved3;    // Byte padding for word alignment

  struct NeoFoxParams_Data  NeoFoxParams;      // NeoFox Parameters

  // Trailer
  unsigned char         Reserved3;         // Byte padding for word alignment
  unsigned char         Reserved4;         // Byte padding for word alignment

  unsigned char         Checksum;          // Checksum of data
  unsigned char         Eof;               // End of frame marker
};

//------------------------------------------------------------------
// Host Initiated Download
//------------------------------------------------------------------
struct ProtNeoFoxDownload
{
  // Header
  unsigned char     Stx;
  unsigned char     PacketType;
  unsigned short    FrameSize;

  // Frame Specific Data
  unsigned long     CmdNumber;   // Host Command Number
  unsigned long     DownloadSizeBytes;
  unsigned long     DownloadTotalFrames;
  unsigned long     DownloadFrameNum;
  unsigned long     DownloadFrameSizeBytes;
  
  unsigned char     DownloadData[MAX_DOWNLOAD_SIZE];

  // Trailer
  unsigned char     Reserved1;   // Byte padding for word alignment
  unsigned char     Reserved2;   // Byte padding for word alignment
  unsigned char     Checksum;
  unsigned char     Eof;
};

#endif

